from matplotlib_venn import venn3, venn3_circles
from matplotlib import pyplot as plt


def complement(subs=1, size=15, fill_color='skyblue', bg_color='white', font_color='#000',
               font_size=20, title_a='Complement A', title_b="Complement A'",
               set_a="A", set_b="A'"):
    """
    Complement Venn diagram

    parameters
    ----------
    subs: 
        1: default(both)
        2: A filled  
        3: A' filled
    size: default 15
    fill_color: default 'skyblue'
    bg_color: default 'white'
    font_color: default '#000' (black)
    font_size: default 20
    title_a: 'default Complement A'
    title_b: "default Complement A'"
    set_a: default 'A'
    set_b: default 'B'

    example
    -------
    complement()
    complement(subs=2, size=5, fill_color='#3eacb5', bg_color='#c1d9db', font_color='#d40f19', 
           font_size=25, title_a='Complement P', set_a='P', set_b="P'")

    """

    if subs == 1:
        figure, (ax1, ax2) = plt.subplots(1, 2, figsize=(size, size))
    elif subs == 2:
        figure, ax1 = plt.subplots(1, 1, figsize=(size, size))
    else:
        figure, ax2 = plt.subplots(1, 1, figsize=(size, size))

    if subs == 1 or subs == 2:
        # A filled
        v1 = venn3(subsets=(1, 1, 0, 1, 0, 0, 0),
                   set_labels=('', '', ''), ax=ax1)
        c1 = venn3_circles(subsets=(1, 1, 0, 1, 0, 0, 0), ax=ax1)
        c1[0].set_lw(0)
        c1[2].set_lw(0)

        for area in ['001', '100']:
            v1.get_patch_by_id(area).set_color(bg_color)
            txt = v1.get_label_by_id(area)
            if txt:
                txt.set_text('')

        v1.get_patch_by_id('010').set_color(fill_color)
        v1.get_patch_by_id('010').set_alpha(1)
        v1.get_label_by_id('010').set_text(set_a)
        v1.get_label_by_id('010').set_fontsize(font_size)
        v1.get_label_by_id('010').set_color(font_color)
        v1.get_label_by_id('001').set_text(set_b)
        v1.get_label_by_id('001').set_fontsize(font_size)
        v1.get_label_by_id('001').set_color(font_color)

        ax1.set_axis_on()
        ax1.set_facecolor(bg_color)
        ax1.text(-1, 0.2, r'U', fontsize=font_size)
        ax1.set_title(title_a, fontsize=font_size)

    if subs == 1 or subs == 3:
        # A' filled
        v2 = venn3(subsets=(1, 1, 0, 1, 0, 0, 0),
                   set_labels=('', '', ''), ax=ax2)
        c2 = venn3_circles(subsets=(1, 1, 0, 1, 0, 0, 0), ax=ax2)
        c2[0].set_lw(0)
        c2[2].set_lw(0)

        for area in ['001', '100']:
            v2.get_patch_by_id(area).set_color(fill_color)
            txt = v2.get_label_by_id(area)
            if txt:
                txt.set_text('')

        v2.get_patch_by_id('010').set_color(bg_color)
        v2.get_patch_by_id('010').set_alpha(1)
        v2.get_label_by_id('010').set_text(set_a)
        v2.get_label_by_id('010').set_fontsize(font_size)
        v2.get_label_by_id('010').set_color(font_color)
        v2.get_label_by_id('001').set_text(set_b)
        v2.get_label_by_id('001').set_fontsize(font_size)
        v2.get_label_by_id('001').set_color(font_color)

        ax2.set_axis_on()
        ax2.set_facecolor(fill_color)
        ax2.text(-1, 0.2, r'U', fontsize=font_size)
        ax2.set_title(title_b, fontsize=font_size)

    plt.show()



from .a_b import a_b
from .and_nand import and_nand
from .complement import complement
from .implication import implication
from .mut_exclusive import mut_exclusive
from .not_not import not_not
from .or_nor import or_nor
from .subsets import subsets
from .true_false import true_false
from .x_not_y import x_not_y
from .xor_xnor import xor_xnor



from matplotlib_venn import venn2, venn2_circles
from matplotlib import pyplot as plt


def not_not(subs=1, size=15, fill_color='skyblue', bg_color='white', font_size=20,
            title_a='NOT A', title_b='NOT B', set_a='A', set_b='B'):
    """
    NOT A, NOT B Venn diagram

    parameters
    ----------
    subs: 
        1: default(both)
        2: NOT A 
        3: NOT B
    size: default 15
    fill_color: default 'skyblue'
    bg_color: default 'white'
    font_size: default 20
    title_a: default 'A'
    title_b: default 'B'
    set_a: default 'A'
    set_b: default 'B'

    example
    -------
    not_not()
    not_not(subs=3, size=5, fill_color='#e8e815', bg_color='#f2f2aa', font_size=25, 
        title_b='NOT Q', set_a='P', set_b='Q')

    """

    if subs == 1:
        figure, (ax1, ax2) = plt.subplots(1, 2, figsize=(size, size))
    elif subs == 2:
        figure, ax1 = plt.subplots(1, 1, figsize=(size, size))
    else:
        figure, ax2 = plt.subplots(1, 1, figsize=(size, size))

    if subs == 1 or subs == 2:
        # NOT A
        ax1.set_title(title_a, fontsize=font_size)

        v1 = venn2(subsets=(3, 3, 1), set_labels=(set_a, set_b), ax=ax1)
        c1 = venn2_circles(subsets=(3, 3, 1), ax=ax1)
        ax1.set_axis_on()
        ax1.set_facecolor(fill_color)
        ymin, ymax = ax1.get_ylim()
        ax1.set_ylim(ymin - 0.1, ymax)

        for area in ['01', '10', '11']:
            color = fill_color if area == '01' else bg_color
            v1.get_patch_by_id(area).set_color(color)
            v1.get_patch_by_id(area).set_alpha(1)
            txt = v1.get_label_by_id(area)
            if txt:
                txt.set_text('')

    if subs == 1 or subs == 3:
        # NOT B
        ax2.set_title(title_b, fontsize=font_size)

        v2 = venn2(subsets=(3, 3, 1), set_labels=(set_a, set_b), ax=ax2)
        c2 = venn2_circles(subsets=(3, 3, 1), ax=ax2)
        ax2.set_axis_on()
        ax2.set_facecolor(fill_color)
        ymin, ymax = ax2.get_ylim()
        ax2.set_ylim(ymin - 0.1, ymax)

        for area in ['01', '10', '11']:
            color = fill_color if area == '10' else bg_color
            v2.get_patch_by_id(area).set_color(color)
            v2.get_patch_by_id(area).set_alpha(1)
            txt = v2.get_label_by_id(area)
            if txt:
                txt.set_text('')

    plt.show()



from matplotlib_venn import venn2, venn2_circles
from matplotlib import pyplot as plt


def mut_exclusive(size=15, fill_color='skyblue', bg_color='white', font_size=20,
                  title='Mutually exclusive: A∩B=∅', set_a='A', set_b='B'):
    """
    Mutually exclusive Venn diagram

    parameters
    ----------
    size: 15
    fill_color: default 'skyblue'
    bg_color: default 'white'
    font_size: default 20
    title: default 'Mutually exclusive: A∩B=∅'
    set_a: default 'A'
    set_b: default 'B'

    example
    -------
    mut_exclusive()
    mut_exclusive(size=10, fill_color='#2d5c91', bg_color='#e1e8f0', font_size=25, 
        title='Mutually exclusive: P∩Q=∅', set_a='P', set_b='Q')

    """

    figure, ax = plt.subplots(1, 1, figsize=(size, size))

    ax.set_title(title, fontsize=font_size)

    v = venn2(subsets=(3, 3, 0), set_labels=(set_a, set_b))
    c = venn2_circles(subsets=(3, 3, 0))
    ax.set_axis_on()
    ax.set_facecolor(bg_color)
    ymin, ymax = ax.get_ylim()
    ax.set_ylim(ymin - 0.1, ymax)

    for area in ['01', '10', '11']:
        if area != '11':
            v.get_patch_by_id(area).set_color(fill_color)
            v.get_patch_by_id(area).set_alpha(1)
        txt = v.get_label_by_id(area)
        if txt:
            txt.set_text('')

    plt.show()



from matplotlib_venn import venn2, venn2_circles
from matplotlib import pyplot as plt


def subsets(size=15, fill_color='skyblue', bg_color='white', font_size=20,
            title='Subsets', set_a='A', set_b='B'):
    """
    Subsets Venn diagram

    parameters
    ----------
    size: default 15
    fill_color: default 'skyblue'
    bg_color: default 'white'
    font_size: default 20
    title: default 'Subsets'
    set_a: default 'A'
    set_b: default 'B'

    example
    -------
    subsets()
    subsets(size=5, fill_color='#f5b705', bg_color='#f7edd0', font_size=20,
        title='Subsets of P', set_a='P', set_b='Q')

    """

    v = venn2(subsets=(5, 0, 2), set_labels=(set_a, set_b))
    c = venn2_circles(subsets=(5, 0, 2))

    for area in ['01', '10', '11']:
        v.get_patch_by_id(area).set_color(fill_color)
        v.get_patch_by_id(area).set_alpha(1)
        txt = v.get_label_by_id(area)
        if txt:
            txt.set_text('')

    plt.text(-0.6, 0.5, r'U', fontsize=font_size)
    plt.gca().set_axis_on()
    plt.gca().set_facecolor(bg_color)
    plt.title(title, fontsize=font_size)
    ymin, ymax = plt.gca().get_ylim()
    plt.ylim(ymin - 0.1, ymax)
    plt.show()
